## Correction devices

As they are started and configured by the manager, correction devices for big detectors need not be in the Karabo project.
Similarly, most settings you will want to change on correction devices should be set via the manager in order to keep them consistent.

The overview scene for a single correction device can, however, be useful for troubleshooting:

![](../static/screenshot-correction-overview.png "Screenshot: Correction device overview scene")

Observations about the state of the correction device at the time this screenshot was taken:

- The `deviceInternalsState` [warning lamp](#warning-lamps) is on
    - At the time the screenshot was taken, this was due to a benign issue connecting to timeserver
- The last `status` was a warning about input hash having unknown source
    - Given that `inputDataState` is no longer red, this was not an issue any longer for the last train processed
- All constants are loaded
    - All correction steps are available
    - All correction steps except `forceMgIfBelow` and `forceHgIfBelow` are enabled (those two are off by default)

### Single module preview

For big detectors, one will usually be interested in the full assembled previews from the [preview assemblers](matchers.md#preview-assemblers).
One can, however, also directly access the preview data from a single correction device.
This is found under the `preview` node which contains one subnode for each preview output of the correction device.
Preview outputs vary from device class to device class (so from detector to detector), but always include raw and corrected.

Within the subnode for a preview output, there is the actual output channel, `preview.[preview type].output` and some configuration settings applicable to the preview; see [preview configuration](../concepts.md#preview).
Note that which options (like image downsampling, reduction across frames, frame selection etc.) are available depends on the detector and preview type; not all settings apply to all types of preview.

Important note: these settings are configured individually per preview channel.
If, for example, you wish to compare two different preview outputs for an XTDF detector pipeline, make sure to set the same frame selection parameters (`index` in particular) on each.

The default correction device scene includes the corrected preview output and links to individual scenes with any additional preview channels.

Single module previews are primarily intended for use in single-module detector installations such as JF500K.
They can also be used to inspect single modules in multi-module detector setups, but one should avoid tweaking individual module preview settings in this case.
Changing the GUI-related settings (flipping along SS / FS axes, downsampling) on the correction device level should be expected to break or render inconsistent the assembled preview.

### Warning lamps

The warning lamps discussed here are displayed with names on the [correction device overview scene](#correction-devices) and shown in the correction device overview of the [manager overview scene](middlelayer.md#calibration-manager).
They are intended to quickly notify the operator of issues with the pipeline.
There are multiple lamps (rather than the regular device state going to `ERROR`) in order to give a hint as to where the errors may originate.
The lamps and their error types are currently:

`inputDataState` indicates a problem with the latest input received - can flicker if detector, DAQ, or network is unreliable

: `EMPTY_HASH` is most commonly encountered when the DAQ is monitoring, but not getting data from the detector.
In that setting, the DAQ will still send data to the pipeline, but it will be "empty".
Specifically, this warning is triggered when opening image data causes a `RuntimeError` because the number of bytes in the array does not match the shape.

: `TRAIN_ID` means that the train ID of the input seems fishy.
Note that while the correction devices do not care about train IDs, train ID issues will affect matching later in the pipeline.
Train IDs are fishy if they are much greater than "current" train ID (see [`trainFromFutureThreshold`](../schemas/BaseCorrection.md#workarounds.trainFromFutureThreshold)) or if they are not monotonic.
In the former case, input is dropped, while in the latter, the warning is just issued (and the [received train percentage](../schemas/BaseCorrection.md#performance.ratioOfRecentTrainsReceived) is reset).

: `MISC_INPUT_DATA` covers other issues such as input source not being in [`fastSources`](../schemas/BaseCorrection.md#fastSources) or input missing image data node.
These issues seem to sometimes show for a single train when DAQ cycles through states.

`processingState`

: `MEMORY_CELL_RANGE` means that the memory cell range for the input (for XTDF detectors, the values in `image.cellId`) exceed the number of memory cells as used for constant queries (`constantParameters.memoryCells`).
Most corrections are per pixel and per memory cell, so frames from cells outside the range of currently loaded constants may not get corrected.
Make sure to set the constant parameters according to the current detector configuration.

: `FRAME_FILTER` means an error occurred during application of the frame filter.

: `PREVIEW_SETTINGS` means that there was a warning when picking the frame for preview, probably because the requested frame was out of bounds or the cell / pulse requested was not found.
The warning text should give some details.

: In future versions of `calng`, there will likely be a catch-all `MISC_PROCESSING` to gather uncaught exceptions in the detector-specific correction code.

`deviceInternalsState` indicates problems within the correction device expected to prevent normal function.

: `CALCAT_CONNECTION` means that the device failed to connect to [CalCat](https://in.xfel.eu/calibration).
In this case, the device will be unable to retrieve constants.

: `TIMESERVER_CONNECTION` means that the device (when doing `getActualTimestamp`) thought that the "current train ID" was zero, typically indicating that the device was not connected to the timeserver.
In this case, the correction device may be unable to preemptively discard erroneous "future train IDs" (see [`trainFromFutureThreshold`](../schemas/BaseCorrection.md#workarounds.trainFromFutureThreshold) and the `TRAIN_ID` warning above).
This does not impair processing, but if "future train IDs" are expected and this warning persists, one should try to fix the configuration.

: `CORRECTION_RUNNER` means that data was received while the correction kernel(s) were not ready.
Some reconfiguration (for instance, changing number of cells on the detector) may trigger recompilation of kernels, causing the lamp to blink briefly.
While this warning is on, the device will be unable to correct data - if it is on and persists for a while after a restart, it is likely an issue for CAL OCD.

: `FRAME_FILTER` - when applying frame filter fails for a given train, this sets a warning on `processingState`.
As it is possible to specify "obviously" invalid frame filters (ex. with indices out of range for the number of frames on input), this warning can additionally be set here on `deviceInternalsState` after reconfiguration.

If an issue is detected, a warning is issued once - this goes in the device `status` and in the log - and the appropriate lamp is turned on.
The lamp is turned back off when the issue is noticed to be resolved - for example, the lamp warning about `inputDataState` might turn on if corrupt data is received for one train, but turned off again if the next train is valid.

Pro tip: you can use Karabo's device property history feature to see recent history of device `status`.
Select a correction device, double-click the "Current value on device" for "Status", select a time range, and "Request History".
This is useful to see why a warning lamp is on in cases where it has been on for a while and other `status` messages might have come since.

### Constant status

For each calibration constant, the correction device exposes a state field to indicate whether the constant was successfully found and loaded.
These are somewhat simpler than the warning lamps described in the previous section:

- Initially, all constants are in the `OFF` state
- Every time a constant is queried / reloaded, the state changes to:
    - `ON` if no exceptions happened
	- `ERROR` in case anything went wrong


Consult the correction device status log for information about what specifically went wrong.
A common source of issues is incorrectly set operating condition parameters, leading to "condition not found" when querying CalCat.
