These are the middlelayer devices typically accompanying calibration pipelines.
They naturally run on a separate device server - a big detector pipeline typically has multiple correction servers (bound), a preview server (bound), and a single middlelayer server.
The middlelayer devices are usually not restarted along with the pipeline, but rather stay around.

## Calibration manager

Responsible for instantiating, configuring, and restarting most other devices, the manager is the primary way to interact with the pipeline.
Its basic configuration is saved in the pipeline Karabo project.
Note that a lot of the manager's schema (including constant retrieval operating condition parameters) is dynamic and so will reset to default when the manager is restarted.

While the pipeline is running, the manager provides convenient reconfiguration of multiple devices simultaneously.
For instance, it allows simultaneously setting the [DAQ train stride](../data-rates.md) on all DAQ devices setting change any of a number of settings on all correction devices.
Thus, although a number of points below (such as [preview configuration](#assembled-preview-configuration) [retrieving constants](#retrieving-constants)) technically deal with settings on correction devices, these should almost always be set through the manager.

![](../static/screenshot-manager-overview.png "Screenshot: Manager overview scene")

### Group configuration

For big multi-module detectors, correction devices are split across multiple compute nodes.
We use the term *group* here to refer to a set of modules handled by a single node.
For AGIPD / DSSC / LPD, a typical calng pipeline will have four groups of four modules - note that groups do not have to correspond to to quadrants.

Module grouping is configured via the [Modules](../schemas/CalibrationManager.md#modules) table.
This table must list all the detector modules with virtual names and each is assigned a group number (the remaining fields in each row typically left blank as they can be inferred).
Modules with the same group number are processed on the same device server - which device server is seen in the [Module groups](../schemas/CalibrationManager.md#moduleGroups) table, which also has some options for automatically starting [group matchers](matchers.md#group-matcher) and setting up their [karabo bridge output](trainmatcher.md#karabo-bridge-output).

### Assembled preview configuration

As illustrated in the simplified overview, previews use a separate data flow.
The main feature of this is to slice or summarize big detector data down to one frame already on the correction devices.
Therefore, settings related to how this is done should typically be managed via the manager to keep settings consistent.

See [preview configuration](../concepts.md#preview) for details on the settings relevant to preview configuration.
Note that the manager will configure the correction devices' frame reduction settings; the settings on the assemblers (such as downsampling / `nan` replacement) are not typically touched and the manager does not set these.
Note also that there will be one [preview assembler](matchers.md#preview-assemblers) per [preview channel](correction-devices.md#single-module-preview) and each preview channel has its own, independent, configuration.

### Retrieving constants

For correction devices to apply their corrections, they need to load appropriate correction constants.
To do so, they need a number of parameters about current operating conditions in order to query [CalCat](https://in.xfel.eu/calibration/).
Through the manager - see the "Constant retrieval parameters" box in the screenshot - these parameters are set and propagated to the correction devices.

Note that one needs to manually update these when changing detector configuration.
Taking the AGIPD example in the screenshots, this typically means ensuring that memory cells, bias voltage, acquisition rate, and gain setting / mode corresponds to the current state of the detector.
Efforts are being made to automate this process.

Once parameters are set, click "Load most recent constants" to start the constant loading process.
The lamps on the manager overview gives an indication of the progress.
The individual correction provide additional information about what was found by the latest query, used to troubleshoot [constant loading issues](../troubleshooting.md#cannot-load-new-constants).

### Restoring device settings on restart

In some cases, you may want to configure running manager-started devices (ex. group matchers) beyond settings maanged directly by the manager (ex. additional data sources).
One way to persist such settings is to add such devices to the project.
Then, however, these devices must be started manually for project-level custom settings to take effect - the manager does not instantiate via project settings.

The ["Restored configurations"](../schemas/CalibrationManager.md#restoredConfigurations) table offers an alternate solution.
Each row in this table specifies a device and key regex.
When instantiating devices, for each row where the device regex matches the device name, the manager will use the Karabo "get configuration from past" feature with the time set to "now" in order to get the last known values for all properties matching the key regex.
This allows live reconfigured settings to be persisted across pipeline restarts.
Make sure to save the manager configuration to the project, though.

## Geometry devices

For multi-module detector setups, a geometry device is typically set up to inform the [preview assemblers](matchers.md#preview-assemblers) about the current geometry of the detector.
The geometry device should be saved in the Karabo project.
It should not need to be restarted and is not managed as such by the manager.

![](../static/screenshot-agipd_geometry.png "Screenshot: AGIPD geometry device scene")

As the above screenshot suggests, the current geometry can be set manually via quadrant positions (or module list, in the case of JUNGFRAU) or loaded from a file.
Geometry devices wrap geometry classes from [EXtra-geom](https://extra-geom.readthedocs.io/en/latest/).

## Condition watchers

As described above, [retrieving constants](#retrieving-constants) requires querying with current operating conditions.
In many cases, this means repeating values set by (a detector) control device(s).
A *condition* device can partly automate the task of keeping the query values in sync.
Still a work in progress, condition devices are available for AGIPD and JUNGFRAU.

![](../static/screenshot-agipd-condition-checker.png "Screenshot: Preliminary condition checker scene (AGIPD)")

The above screenshot from the `calng` installation at MID reveals some details:

- The condition checker distinguishes between the current value on the control device, the ideal value that the manager should use for constant querying based on that, and the current value actually set on the manager
    - For some values like gain mode, the condition device maps from an integer control value (`gainModeIndex`) to the descriptive enum used in `calng` (`AgipdGainMode` with members like `ADAPTIVE_GAIN`)
    - For other values like bias voltage, the condition device rounds the floating control value to avoid small fluctuations interfering with constant retrieval
- Some parameters - like photon energy - are not found on detector control device which is indicated by `IGNORING`
    - In the screenshot, this includes `pixelsX` and `pixelsY` which are anyway fixed for the detector
    - It additionally shows `deviceMappingSnapshotAt` and `constantVersionEventAt` which are [CalCat](https://in.xfel.eu/calibration/)-specific parameters only relevant for certain testing scenarios and should generally be left blank for normal operation

## ROI tool

The ROI tool is a small device developed to help keep track of pixel intensities.
It is not very fast and is usually just connected to a preview output of corrected data.
Note that using [preview data](../concepts.md#preview) for any kind of analysis comes with a lot of caveats and should generally be avoided.

With these limitations in mind, the purpose of the ROI tool is to just monitor the intensity distribution in some region of interest.
The device scene will give you a widget to select the ROI, show you a histogram of a running average distribution within this ROI, and some options to tweak the histogramming.

Currently deployed for testing at SPB; see `SPB_DET_AGIPD1M-1/CALNG/ROI_HISTOGRAM_CORR` and `SPB_IRDA_JF4M/CALNG/ROI_TOOL_CORR`.
