# DetectorAssembler
Base classes: [TrainMatcher](TrainMatcher.md)
## <a name='timeOfFlight'></a>Time of flight (`timeOfFlight`)
Type
: DOUBLE

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0.0``

## <a name='processingTime'></a>Processing time (`processingTime`)
Type
: DOUBLE

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0.0``

## <a name='imageDataPath'></a>`imageDataPath`
Type
: STRING

Access mode
: INITONLY

Assignment
: OPTIONAL

Default value
: ``image.data``

## <a name='imageMaskPath'></a>`imageMaskPath`
Type
: STRING

Access mode
: INITONLY

Assignment
: OPTIONAL

Default value
: ``image.mask``

## `assembledOutput`
OutputChannel
## <a name='outputForBridgeOutput'></a>Bridge output (`outputForBridgeOutput`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: The bridge can send the same data as any one of the three Karabo output channels. If this setting is 'matched', it will follow 'output', if 'assembled', it will follow 'assembledOutput' and if 'preview', it will follow 'preview.output'.

Options:
: ``matched``, ``assembled``

Default value
: ``matched``

## <a name='geometryDevice'></a>Geometry device (`geometryDevice`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: The the device which will provide geometry. The device is expected to have current geometry as a string (for now using custom non-robust serialization scheme) element named serializedGeometry.

## <a name='sourceIndexPattern'></a>Source index regex (`sourceIndexPattern`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Regular expression used to extract the module index (used with geometry for assembly) from source names. This must contain a capture group named 'index'; an example expression could be '.*\/MY_DETECTOR\/(?P<index>\d+).*' (note the '.*' in the prefix; the expression must match from the start of the source name). If no source index pattern is provided, some default patterns for correction devices, XTDF detectors, or DAQ outputs (as typically used with JUNGFRAU detectors) will be tried.

Default value
: empty string

## <a name='sourceToModuleIndex'></a>Module index mapping (`sourceToModuleIndex`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: This table shows how the sourceIndexPattern (or if not set, one of the default patterns) extracted module indices for the configured (and selected) sources. Empty indices indicate sources which did not match the regular expression and will not be used in assembly.

Table schema and default value
: 
| select | source | regexUsed | index |
| - | - | - | - |


## Preview (`preview`)
Output specifically intended for preview in Karabo GUI. Includes some options for throttling and adjustments of the output data.
### `preview.assembled`
#### <a name='preview.assembled.swapAxes'></a>Swap slow / fast scan (`preview.assembled.swapAxes`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Swaps the two pixel axes around. Can be combined with flipping to rotate the image

Default value
: ``False``

#### <a name='preview.assembled.flipSS'></a>Flip SS (`preview.assembled.flipSS`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Flip image data along slow scan axis.

Default value
: ``True``

#### <a name='preview.assembled.flipFS'></a>Flip FS (`preview.assembled.flipFS`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Flip image data along fast scan axis.

Default value
: ``True``

#### <a name='preview.assembled.downsamplingFactor'></a>Downsampling factor (`preview.assembled.downsamplingFactor`)
Type
: UINT32

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: If greater than 1, the preview image will be downsampled by this factor before sending. This is mostly to save bandwidth in case GUI updates start lagging.

Options:
: ``1``, ``2``, ``4``, ``8``

Default value
: ``1``

#### <a name='preview.assembled.downsamplingFunction'></a>Downsampling function (`preview.assembled.downsamplingFunction`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Reduction function used during downsampling.

Options:
: ``nanmax``, ``nanmean``, ``nanmin``, ``nanmedian``

Default value
: ``nanmax``

#### <a name='preview.assembled.replaceNanWith'></a>NaN replacement (`preview.assembled.replaceNanWith`)
Type
: FLOAT

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Displaying images in KaraboGUI seems to not go well when there are NaN values in data. And there will be with bad pixel masking or just geometry space between modules. NaN values get replaced with this value to get around this; choose a value which clearly stands out from the image data you want to see.

Default value
: ``0.0``

#### `preview.assembled.output`
See description of grandparent node, 'preview'.
OutputChannel