# Epix100Geometry
Base classes: ManualOriginGeometryBase
## <a name='availableScenes'></a>Available scenes (`availableScenes`)
Type
: VECTOR_STRING

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``['overview']``

## Geometry preview (`geometryPreview`)
ImageData
## Geometry file (`geometryFile`)
Allows loading geometry from CrystFEL geometry file
### <a name='geometryFile.filePath'></a>File path (`geometryFile.filePath`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Full path (including filename and suffix) to the desired geometry file. Keep in mind that the default directory is $KARABO/var/data on device server node, so it's probably wise to give absolute path.

Default value
: empty string

### <a name='geometryFile.fileType'></a>File type (`geometryFile.fileType`)
Type
: STRING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: What kind of file will be loaded. Corresponds to options within EXtra-geom. Note that the options listed here may not all apply to all geometries. I think 'crystfel' (uses extra_geom.[geometry type].from_crystfel_geom is the most 'universal', so I left that as default.

Options:
: ``crystfel``, ``h5``, ``h5+quadrants``

Default value
: ``crystfel``

### Offset (`geometryFile.offset`)
See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.
#### <a name='geometryFile.offset.x'></a>`geometryFile.offset.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

#### <a name='geometryFile.offset.y'></a>`geometryFile.offset.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='geometryFile.offset'></a>Offset (`geometryFile.offset`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.

### <a name='geometryFile.updateManualOnLoad'></a>Update manual settings (`geometryFile.updateManualOnLoad`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: If this flag is on, the manual settings on this device will be updated according to the loaded geometry file. This is useful when you want to load a file and then tweak the geometry a bit. This will zero current offset.

Default value
: ``True``

### <a name='geometryFile.updateMotorReferenceOnLoad'></a>Update motor reference (`geometryFile.updateMotorReferenceOnLoad`)
Type
: BOOL

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: If this flag is on, the motor reference positions in the motor tracking node will be updated according to the motor positions stored in geometry file.

Default value
: ``True``

### <a name='geometryFile.loadFromFile'></a>Load from file (`geometryFile.loadFromFile`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='geometryFile'></a>Geometry file (`geometryFile`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Allows loading geometry from CrystFEL geometry file

## Tweak geometry (`tweakGeometry`)
### <a name='tweakGeometry.undo'></a>Undo (`tweakGeometry.undo`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### <a name='tweakGeometry.redo'></a>Redo (`tweakGeometry.redo`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### Offset (`tweakGeometry.offset`)
See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.
#### <a name='tweakGeometry.offset.x'></a>`tweakGeometry.offset.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

#### <a name='tweakGeometry.offset.y'></a>`tweakGeometry.offset.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='tweakGeometry.offset'></a>Offset (`tweakGeometry.offset`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.

### <a name='tweakGeometry.add'></a>Add offset (`tweakGeometry.add`)
Type
: Slot

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### <a name='tweakGeometry.undoLength'></a>Undo length (`tweakGeometry.undoLength`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0``

### <a name='tweakGeometry.redoLength'></a>Redo length (`tweakGeometry.redoLength`)
Type
: INT32

Access mode
: READONLY

Assignment
: OPTIONAL

Default value
: ``0``

## <a name='tweakGeometry'></a>Tweak geometry (`tweakGeometry`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='sendGeometry'></a>Send geometry (`sendGeometry`)
Type
: Slot

Allowed in states
: ACTIVE, MONITORING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Will send 'signalNewGeometry' to connected slots. These will for example be DetectorAssembler. Note that signal is sent automatically when new geometry is set - this slot is mostly to be called by manager after (re)starting assemblers while geometry device is still up.

## <a name='updatePreview'></a>Update preview (`updatePreview`)
Type
: Slot

Allowed in states
: ACTIVE, MONITORING

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## `origin`
### Offset (`origin.offset`)
See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.
#### <a name='origin.offset.x'></a>`origin.offset.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

#### <a name='origin.offset.y'></a>`origin.offset.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='origin.offset'></a>Offset (`origin.offset`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: See EXtra-geom documentation for details. This offset is applied to entire detector after initial geometry is created from manual parameters. Example: To move entire geometry up by 2 mm relative to beam, set offset.y to 2e-3.

### <a name='origin.setManual'></a>Set manual geometry (`origin.setManual`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

### <a name='origin.x'></a>`origin.x`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

### <a name='origin.y'></a>`origin.y`
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Default value
: ``0.0``

## <a name='origin'></a>`origin`
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL
