from karabo.bound import (
    DOUBLE_ELEMENT,
    Unit,
)
import numpy as np
from .base_kernel import BaseArbiterKernel


class RandomSampler(BaseArbiterKernel):
    @staticmethod
    def extend_device_schema(schema, prefix):
        (
            DOUBLE_ELEMENT(schema)
            .key(f"{prefix}.probability")
            .unit(Unit.PERCENT)
            .assignmentOptional()
            .defaultValue(50)
            .reconfigurable()
            .commit(),
        )

    def consider(self, train_id, sources, num_frames, mask, out_hash):
        return mask & (np.random.random(num_frames) > self._threshold)

    def reconfigure(self, config):
        if config.has("probability"):
            self._threshold = (100 - config.get("probability")) / 100
