class BaseCorrectionAddon:
    _device = None  # will be set to host device *after* init

    @staticmethod
    def extend_device_schema(schema, prefix):
        """Will be given the device schema where everything should be put under a
        prefix. This prefix should be something like 'addons.nodeName', is given to this
        class by the base correction device, and the root node of prefix will already be
        in place (as will an 'enable' flag under it)."""
        pass

    @staticmethod
    def extend_output_schema(schema):
        """Will be given the regular output schema, can create arbitrary nodes in there
        and add properties to it."""
        pass

    def __init__(self, config):
        """Will be given the node from extend_device_schema, no prefix needed here"""
        pass

    def post_correction(
        self, train_id, processed_data, cell_table, pulse_table, output_hash
    ):
        """Called directly after correction has happened. Processed data will still be
        on GPU if the correction device is generally running in GPU mode."""
        pass

    def post_reshape(
        self, train_id, reshaped_data, cell_table, pulse_table, output_hash
    ):
        pass

    def reconfigure(self, changed_config):
        """Will be given the node from extend_device_schema, no prefix needed here. Note
        that only the changed properties will be in this update hash; see peakfinder9
        for an example of caching and reinstanting complex kernels using many parameters
        when any change."""
        pass

    @property
    def geometry(self):
        """Helper to get current geometry from host device. If addon will do expensive
        precomputation only when geometry changes, use the on_new_geometry hook to
        trigger this."""
        return self._device._geometry

    def on_new_geometry(self, geometry):
        """Hook which is called whenever the host device receives a new geometry from
        the geometry device. Useful to know when geometry changes in case you plan to
        do expensive precomputation based on it (ex. azimuthal integration)."""
        pass
