# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

cdef unsigned char NONE = 0
cdef unsigned char OFFSET = 1
cdef unsigned char REL_GAIN = 2
cdef unsigned char BPMASK = 4
cdef unsigned char STRIXEL = 8

from cython.parallel import prange
from cython.view cimport contiguous
import numpy as np


def correct_burst(
    unsigned short[:, :, :] image_data,
    unsigned char[:, :, :] gain_stage,
    unsigned char[:] cell_table,
    unsigned char flags,
    float[:, :, :, :] offset_map,
    float[:, :, :, :] relgain_map,
    unsigned[:, :, :, :] badpixel_mask,
    float badpixel_fill_value,
    float[:, :, :] output,
):
    cdef int frame, map_cell, fs, ss
    cdef unsigned char gain
    cdef float corrected
    for frame in prange(image_data.shape[0], nogil=True):
        map_cell = cell_table[frame]
        if map_cell >= offset_map.shape[0]:
            for ss in range(image_data.shape[1]):
                for fs in range(image_data.shape[2]):
                    output[frame, ss, fs] = <float>image_data[frame, ss, fs]
            continue
        for ss in range(image_data.shape[1]):
            for fs in range(image_data.shape[2]):
                corrected = image_data[frame, ss, fs]
                gain = gain_stage[frame, ss, fs]
                # legal values: 0, 1, or 3
                if gain == 2:
                    corrected = badpixel_fill_value
                else:
                    if gain == 3:
                        gain = 2

                    if (flags & BPMASK) and badpixel_mask[map_cell, ss, fs, gain] != 0:
                        corrected = badpixel_fill_value
                    else:
                        if (flags & OFFSET):
                            corrected = corrected - offset_map[map_cell, ss, fs, gain]
                        if (flags & REL_GAIN):
                            corrected = corrected / relgain_map[map_cell, ss, fs, gain]
                output[frame, ss, fs] = corrected


def correct_single(
    unsigned short[:, :, :] image_data,
    unsigned char[:, :, :] gain_stage,
    unsigned char flags,
    float[:, :, :, :] offset_map,
    float[:, :, :, :] relgain_map,
    unsigned[:, :, :, :] badpixel_mask,
    float badpixel_fill_value,
    float[:, :, :] output,
):
    # ignore "cell table", constant pretends cell 0
    cdef int fs, ss
    cdef unsigned char gain
    cdef float corrected
    for ss in range(image_data.shape[1]):
        for fs in range(image_data.shape[2]):
            corrected = image_data[0, ss, fs]
            gain = gain_stage[0, ss, fs]
            # legal values: 0, 1, or 3
            if gain == 2:
                corrected = badpixel_fill_value
            else:
                if gain == 3:
                    gain = 2

                if (flags & BPMASK) and badpixel_mask[0, ss, fs, gain] != 0:
                    corrected = badpixel_fill_value
                else:
                    if (flags & OFFSET):
                        corrected = corrected - offset_map[0, ss, fs, gain]
                    if (flags & REL_GAIN):
                        corrected = corrected / relgain_map[0, ss, fs, gain]
            output[0, ss, fs] = corrected
