#include <cuda_fp16.h>

{{corr_enum}}

extern "C" {
	__global__ void correct(const unsigned short* data, // shape: memory cell, y, x
	                        const unsigned char* gain_stage, // same shape
	                        const unsigned char* cell_table,
	                        const unsigned char corr_flags,
	                        const unsigned short input_frames,
	                        const unsigned short map_memory_cells,
	                        const unsigned char burst_mode,
	                        const float* offset_map,
	                        const float* rel_gain_map,
	                        const unsigned int* bad_pixel_map,
	                        const float bad_pixel_mask_value,
	                        {{output_data_dtype}}* output) {
		const size_t ss_dim = 512;
		const size_t fs_dim = 1024;

		const size_t current_frame = blockIdx.x * blockDim.x + threadIdx.x;
		const size_t ss = blockIdx.y * blockDim.y + threadIdx.y;
		const size_t fs = blockIdx.z * blockDim.z + threadIdx.z;

		if (current_frame >= input_frames || ss >= ss_dim || fs >= fs_dim) {
			return;
		}

		const size_t data_stride_fs = 1;
		const size_t data_stride_ss = fs_dim * data_stride_fs;
		const size_t data_stride_frame = ss_dim * data_stride_ss;
		const size_t data_index = current_frame * data_stride_frame +
			ss * data_stride_ss +
			fs * data_stride_fs;
		float res = (float)data[data_index];

		// gain mapped constant shape: cell, y, x, gain_level (dim size 3)
		// note: in fixed gain mode, constant still provides data for three stages
		const size_t map_stride_gain = 1;
		const size_t map_stride_fs = 3 * map_stride_gain;
		const size_t map_stride_ss = fs_dim * map_stride_fs;
		const size_t map_stride_cell = ss_dim * map_stride_ss;

		// TODO: warn user about cell_table value of 255 in either mode
		// note: cell table may contain 255 if data didn't arrive
		// burst mode: "cell 255" will get copied (not corrected)
		// single cell: "cell 255" will get "corrected"
		const size_t map_cell = burst_mode ? cell_table[current_frame] : 0;
		if (map_cell < map_memory_cells) {
			unsigned char gain = gain_stage[data_index];
			if (gain == 2) {
				// gain should be read as 0, 1, or 3; value of 2 indicates issue
				res = bad_pixel_mask_value;
			} else {
				if (gain == 3) {
					// but 3 means 2 in the constants
					gain = 2;
				}
				if (map_cell < map_memory_cells) {
					// only correct if in range of constant data
					const size_t map_index = map_cell * map_stride_cell +
						ss * map_stride_ss +
						fs * map_stride_fs +
						gain * map_stride_gain;
					if ((corr_flags & BPMASK) && bad_pixel_map[map_index]) {
						res = bad_pixel_mask_value;
					} else {
						if (corr_flags & OFFSET) {
							res -= offset_map[map_index];
						}
						if (corr_flags & REL_GAIN) {
							res /= rel_gain_map[map_index];
						}
					}
				}
			}
		}

		{% if output_data_dtype == "half" %}
		output[data_index] = __float2half(res);
		{% else %}
		output[data_index] = ({{output_data_dtype}})res;
		{% endif %}
	}
}
