from karabo.bound import Configurator, Schema

from calng.correction_addons import litpixel_counter, random_frames
from calng.corrections import (
    AgipdCorrection,
    DsscCorrection,
    Epix100Correction,
    Gotthard2Correction,
    JungfrauCorrection,
    LpdCorrection,
    LpdminiCorrection,
    PnccdCorrection,
)


def test_available_addons():
    all_device_classes = [
        AgipdCorrection,
        DsscCorrection,
        Epix100Correction,
        Gotthard2Correction,
        JungfrauCorrection,
        LpdCorrection,
        LpdminiCorrection,
        PnccdCorrection,
    ]
    actual_classes = [
        getattr(cls, cls.__name__.rpartition(".")[-1]) for cls in all_device_classes
    ]

    configurator = Configurator("PythonDevice")
    for cls in actual_classes:
        schema_hash = configurator.getSchema(cls).getParameterHash()

        # everyone should have RandomFrames
        assert random_frames.RandomFrames in cls._available_addons  # available
        assert schema_hash.has("addons.RandomFrames")  # and in schema
        # only AGIPD has the litpixelcounter (for now)
        if cls == AgipdCorrection.AgipdCorrection:
            assert litpixel_counter.LitPixelCounter in cls._available_addons
            assert schema_hash.has("addons.LitPixelCounter")
        else:
            assert litpixel_counter.LitPixelCounter not in cls._available_addons
            assert not schema_hash.has("addons.LitPixelCounter")
